/*----------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich & W.Wallucks
 Remarks:        
 known Problems: none
 Version:        24.10.2007
 Description:    Webserver uvm.

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
----------------------------------------------------------------------------*/

#include <stdlib.h>
#include <stdio.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "config.h"

#include "usart.h"
#include "networkcard/enc28j60.h"
#include "stack.h"
#include "timer.h"
#include "base64.h"
#include "httpd.h"
#include "telnetd.h"


#if USE_NTP
  #include "ntp.h"
#endif


#if USE_ADC
  #include "analog.h"
#endif


#if USE_DNS
  #include "dns.h"
#endif

#if USE_MMC
  #include "sdkarte/fat16.h"
  #include "sdkarte/sdcard.h"
#endif

#if TCP_SERVICE
  #include "tcpservice/tcpcmd.h"
  #include "tcpservice/tcpsrv.h"
#else
  #include "cmd.h"
#endif


/**
 * \mainpage Webmodule
 *
 * In diesem Projekt werden einige \ref mainloop "Erweiterungen zur ETH32-Software" von Ulrich Radig beschrieben.
 * Als Hardware wird das Webmodul von Ulrich Radig (http://www.ulrichradig.de/home/index.php/avr/avr-webmodule)
 * verwendet. Die ursprngliche Software findet sich hier http://www.ulrichradig.de/home/index.php/avr/eth_m32_ex
 *
 * \image html Webmodule.jpg "Ulrich Radigs Webmodul mit SD-Karte"
 *
 *	Des weiteren wurde das FAT16-Filesystem von Roland Riegel verwendet. 
 *	(http://www.roland-riegel.de/sd-reader/index.html)<br>
 * sowie Code von Martin Thomas (1-Wire DS18B20) und Peter Dannegger (1-Wire und RC5)
 */

/**
 * \page hardware	Hardwareanpassung
 *
 * @{
 *	Der SD-Kartenslot aus Ulis Shop hat einen Schalter, der geschlossen ist wenn 
 *	die Karte gesteckt ist. Leider ist der Schalter aber auf der Originalplatine nicht 
 *	verdrahtet.
 *
 *	\b Abhilfe: einen der Anschlusspins vom Slot (befinden sich an der Ecke 
 *	wo der Bestckungsaufdruck MMC1 ist) auf Masse festlten und den anderen an PA6 
 *	mit einem kleinen Draht verbinden. \n
 *	Ohne den Schalter gibt es einen langen Timeout, wenn man auf die SD-Karte
 *	zugreifen will und es ist keine Karte gesteckt. Ebenso ist es durch den Schalter
 *	mglich einen Wechsel der Karte zu erkennen und sie neu zu initialisieren.
 *	Siehe dazu die Funktion \ref f16_check().
 *
 *	Ohne diese Hardwarenderung muss im Quelltext in <em>sd_raw_config.h</em> die Zeile
 *	\code #define get_pin_available() ((PINA & (1<<PA6))) \endcode
 *	nach
 *	\code #define get_pin_available() (1) \endcode
 *	abgendert werden.
 *
 *	Grundstzlich hier noch ein paar Anmerkungen:
 *
 *	\li <b>Die Webseiten welche von der SD-Karte gelesen werden sollen mssen sich alle im
 *	Hauptverzeichnis der Karte befinden.</b> Der Code um Dateien aus verschiedenen
 *	Unterverzeichnissen zusammenzusuchen war mir bisher zu aufwendig und ist fr
 *	einen Miniwebserver wohl auch nicht ntig.
 *
 *	\li Die <b>Anzeige der Dateien mit Datum, Gre, Berechtigungen etc.</b> ist in den RFC
 *	fr FTP nicht definiert. Ich habe bisher keine Ahnung, wie eine allgemeingltige
 *	Verzeichnisanzeige auszusehen hat. Je nach FTP-Client wird daher die
 *	Verzeichnisstruktur unterschiedlich dargestellt. Mit FileZilla funktioniert es
 *	bei mir jedenfalls recht gut.
 *
 *	\li <b>Wie muss man nun vorgehen, wenn man von der SD-Karte eine HTML-Seite laden mchte?</b> \n
 *	html-Datei erstellen und im Root-Verzeichnis der Karte ablegen. Zustzliche Dateien
 *	(Bilder) auch ins Root-Verzeichnis ablegen, da der Code um die Dateien im
 *	Unterverzeichnis zu suchen zuviel Overhead mit sich bringt. Falls man index.htm auf der
 *	Karte abspeichert wird sie direkt aufgerufen. Ansonsten die Datei im Browser
 *	angeben <tt>http://192.168.0.99/default.htm</tt> falls man default.htm abspeichert.
 *
 *	\li <b>Wie stellt man das an, dass Dateien auf der Karte ber das Internet abrufbar sind?</b> \n
 *	Mit FileZilla drauf zugreifen! Wenn man von "auen" auf sein Webmodul zugreifen will,
 *	muss man Dyndns einrichten. In der Fritzbox gibt man dann unter Portfreigabe das
 *	Webmodul nach auen frei. Am Einfachsten setzt man hierzu den Eintrag "Exposed Host"
 *	auf die Adresse seines Webmoduls. Es sollte kein Problem bereiten das Teil komplett
 *	freizugeben. 
 *
 *	\li <b>Kann ich auch vom AVR-Server darauf zugreifen und wie muss man vorgehen um
 *	Dateien zu erzeugen zu lschen zu laden usw. ?</b>
 *	Das Webmodul ist ein FTP-Server. Eine FTP-Client-Funktion ist nicht dabei!
 *	Theoretisch geht das schon wenn man es programmiert.
 *
 *	Praktisch die gesamte Grundkonfiguration des Moduls erfogt in der Datei config.h. Dort
 *	sind je nach Hardwareausstattung und bentigter Funktionalitt einige <tt>define</tt>
 *	vorgegeben, die durch setzen von null(0) oder eins(1) entsprechend aus- oder angeschaltet
 *	werden knnen.
 *
 * @}
 *
 */

/**
 * \defgroup main	Hauptprogramm
 *
 * siehe auch Beschreibung der \ref mainloop "Hauptschleife" weiter unten.
 * @{
 */

/**
 * \file Hauptprogramm
 *
 * \author Ulrich Radig & W.Wallucks
 *
 */

/*
**	globale Variable
*/
volatile ANLAGEN_STATUS anlagenStatus;		//!< aktueller Zustand der Schalter und Zhler
volatile PROZESS_STATUS machineStatus;		//!< Prozessstatus und aktuelle Werte
#if USE_MMC
		 LOG_STATUS		logStatus;			//!< Status der Logdatei
#endif
volatile SOLL_STATUS 	anlagenSollStatus;	//!< Zustandsbeschreibung der angeschlossenen Schalter und Relais

//----------------------------------------------------------------------------
/**
 *	Variablenarry zum Einfgen in Webseite \%VA\@00 bis \%VA\@09<br>
 *	in den ersten Speicherpltzen werden bei aktiviertem ADC die
 *	gemessenen Analogwerte gespeichert
 */
unsigned int var_array[MAX_VAR_ARRAY] = {10,50,30,0,0,0,0,0,0,0};
unsigned int var_projekt[MAX_VARIABLEN] = {0,0,0,0,0,0,0,0,0,0}; //Temperaturwerte/Stati (Projektarbeit)

FILE usart_out = FDEV_SETUP_STREAM(usart_putchar, NULL, _FDEV_SETUP_WRITE);

/**
 **	lokale Prototypen fr Timerfunktionen
*/
void read_T(void);

//----------------------------------------------------------------------------
//
int main(void)
{
	//Konfiguration der Ausgnge bzw. Eingnge
	//definition erfolgt in der config.h
	DDRA = OUTA;
	DDRC = OUTC;
	DDRD = OUTD;
	
    unsigned long a;
	
    usart_init(BAUDRATE); 	// setup the USART
	stdout = &usart_out;	// set standard lib-functions
	
	
	//Statusausgabe
	//printf_P(PSTR("\n\rSystem Ready\n\r"));
    //printf_P(PSTR("Compiliert am "__DATE__" um "__TIME__"\r\n"));
    //printf_P(PSTR("Compiliert mit GCC Version "__VERSION__"\r\n"));
	for(a=0;a<1000000;a++){asm("nop");};

	//Applikationen starten
	timer_init();	// Timer starten - von stack_init() hierher verschoben
	stack_init();
	httpd_init();
	telnetd_init();
	

	//Ethernetcard Interrupt enable
	ETH_INT_ENABLE;
	
	//Globale Interrupts einschalten
	sei(); 
	
	
	#if USE_DNS
		dns_init();
	#endif

	#if USE_NTP
        ntp_init();
	#endif //USE_NTP
    
	#if TCP_SERVICE
		tcpsrvd_init();
	#endif

    #if USE_MMC
		f16_init();
	#endif

	
	 /**
	 * \ingroup main
	 * \anchor mainloop
 	 * \b Hauptschleife
	 *
	 *	Die Hauptschleife wird ber mehrere Flags gesteuert.
	 *  Diese Flags sind in der Struktur #PROZESS_STATUS <tt>machineStatus</tt>
	 *	zusammengefasst.
	 *	Das Setzen der Flags geschieht gewhnlich in Interrupts (Timer, PIN changed etc.)
	 *	oder durch ber-/Unterschreiten von Schwellwerten.
	 *
	 *	Als erstes wird in der Hauptschleife die Zeit hochgezhlt. Der Timerinterrupt
	 *	zhlt nur das Flag #PROZESS_STATUS.timeChanged. Dadurch ist es mglich bei Erreichen
	 *  einer vorgegebenen Schaltzeit auch lngere Aktionen auszufhren ohne auf Dauer eine
	 *	Gangungenauigkeit zu riskieren. Die zwischenzeitlich vergangenen Sekunden werden
	 *	notfalls einzeln in den nchten Schleifendurchgngen beschleunigt nachgeholt.
	 *
	 *	Danach werden gesetzte Flags einzeln abgefragt, eine entsprechende Funktion zum
	 *	Ausfhren der Aktion aufgerufen und das Flag wieder zurckgesetzt.
	 *
	 *	Regelmig wird dabei 
	 *	- die Ethernetschnittstelle mit #eth_get_data() gepollt;
	 *	- die serielle Schnittstelle auf Ende des
	 *	  Befehls abgefragt #usart_status.usart_ready
	 *	- bei TELNET-Verbindung auf Port 23 werden Zeichen
	 *	  von USART->TCP geschickt <tt>telnetd_send_data();</tt>
	 *
	 */	
	 
	 //Startwerte
	 
	while(1)
	{
		if (machineStatus.timeChanged) {
			// eine Sekunde hochzhlen und
			// auf nderungen zu vordefinierten Zeiten reagieren
			TM_AddOneSecond();
			machineStatus.timeChanged--;
			eth.timer = 1;

			// Countdown-Timer bei Bedarf runterzhlen
			if (machineStatus.Timer1) {
				machineStatus.Timer1--;
				if (!machineStatus.Timer1) {	// Zero?
					// bei Zero (0) Flag fr Aktion setzen
					// oder sofort ausfhren
					PORTA &= ~(1<<2);	// Pin 2 resetten
				}
			}

			// ... und der nchste Countdown
			if (machineStatus.Timer2) {
				machineStatus.Timer2--;
				if (!machineStatus.Timer2) {			// Zero?
					if (machineStatus.Timer2_func)
						machineStatus.Timer2_func();	// ausfhren
				}
			}
		}

		/**
		*	\anchor t3timer
		*	<H3>Countdown-Timer</H3>
		*
		*	Timer3 wird in ISR(TIMER1_COMPA_vect) im Takt von TIMERBASE
		*	heruntergezhlt. Bei Erreichen von Null wird das Flag Time3Elapsed
		*	gesetzt, was hier ausgewertet wird. Falls der Funktionszeiger
		*	Timer3_func nicht NULL ist, wird die entsprechende Funktion
		*	aufgerufen.
		*
		*	Beispiel:
		*	\code
		*	if (!machineStatus.Timer3) {			// wenn Timer3 nicht belegt ist ...
		*		machineStatus.Timer3 = start_OW();	// liefert Wartezeit (750 ms) zurck und startet Timer3
		*		machineStatus.Timer3_func = read_T;	// Funktionspointer fr abgelaufene Zeit setzen
		*	}
		*	\endcode
		*
		*/
		if (machineStatus.Time3Elapsed) {
			machineStatus.Time3Elapsed = false;
			if (machineStatus.Timer3_func)
				machineStatus.Timer3_func();			// ausfhren
		}


		#if USE_MMC
		f16_check();	// Ein-/Ausstecken der SD-Karte erkennen
		#endif


		#if USE_PROJEKT
		//berprfen ob Zeit zum Abholen von Werten gekommen (Projektarbeit)
		if (machineStatus.Tlesen) {
			machineStatus.Tlesen = 0;
			read_T();
		}
		#endif

		//Netzwerk pollen
	    eth_get_data();
		
        
		//Terminalcommandos auswerten
		if (usart_status.usart_ready){
            usart_write("\r\n");
			if(extract_cmd(&usart_rx_buffer[0]))
			{
			#if USE_MMC
				usart_write("\r\nSD:%s>",cwdirectory);
			#else
				usart_write("Ready\r\n\r\n");
			#endif
			}
			else
			{
				usart_write("ERROR\r\n\r\n");
			}
			
			
			
			usart_status.usart_ready =0;
		}

        //Empfang von Zeitinformationen
		#if USE_NTP
		if(!ntp_timer){
			ntp_timer = NTP_REFRESH;
			ntp_request();
		}
		#endif //USE_NTP
		
           
		//USART Daten fr Telnetanwendung?
		telnetd_send_data();
    }
	return(0);
}


#if USE_PROJEKT
//Temperaturen/Werte von H_Steuerung und Regler abholen (Projektarbeit)
void read_T(void)
{
	//unsigned long ser_timeout = 0;
	
	//Aufforderungen auf Bus schreiben
	//usart_write("Heizung: tempbrenner11\r\n");
	
	usart_write_rs485("Regler: Wtempraum\r\n\r");
	
	/*
	//Antwort empfangen
	while((!usart_status.usart_ready) && (ser_timeout<1000))
	{
		_delay_ms(1);
		ser_timeout++;
	}
	ser_timeout=0;
		
	usart_write_rs485("Antwort: ");
	usart_write_rs485(usart_rx_buffer);
	usart_write_rs485("\r\n\r\n");
	usart_status.usart_ready=0;
	*/

}
#endif
